@@ -4,6 +4,8 @@
 # This module is used to add extra widgets to the QUISK screen.
 
 from __future__ import print_function
+from __future__ import absolute_import
+from __future__ import division
 
 import math, wx
 
@@ -98,7 +100,7 @@ class BottomWidgets:	# Add extra widgets to the bottom of the screen
   def UpdateText(self):
     # Temperature
     temp = self.Code2Temp()
-    temp = (" Temp %3.0f" % temp) + unichr(0x2103)
+    temp = (" Temp %3.0f" % temp) + u'\u2103'
     self.text_temperature.SetLabel(temp)
     # power amp current
     current = self.Code2Current()
