@@ -1,8 +1,11 @@
 # These are Quisk widgets
 
+from __future__ import print_function
+from __future__ import absolute_import
+from __future__ import division
+
 import sys, re
 import wx, wx.lib.buttons, wx.lib.stattext
-from types import *
 # The main script will alter quisk_conf_defaults to include the user's config file.
 import quisk_conf_defaults as conf
 import _quisk as QS
@@ -16,11 +19,13 @@ def EmptyBitmap(width, height):
     return wx.Bitmap(width, height)
 
 def MakeWidgetGlobals():
-  global button_font, button_bezel, button_width, button_height, button_text_width, button_text_height
+  global button_font, button_uline_font, button_bezel, button_width, button_height, button_text_width, button_text_height
   global _bitmap_menupop, _bitmap_sliderpop, _bitmap_cyclepop
   button_bezel = 3		# size of button bezel in pixels
   button_font = wx.Font(conf.button_font_size, wx.FONTFAMILY_SWISS, wx.NORMAL,
            wx.FONTWEIGHT_NORMAL, False, conf.quisk_typeface)
+  button_uline_font = wx.Font(conf.button_font_size, wx.FONTFAMILY_SWISS, wx.NORMAL,
+           wx.FONTWEIGHT_NORMAL, True, conf.quisk_typeface)
   dc = wx.MemoryDC()
   dc.SetFont(button_font)
   tmp_bm = EmptyBitmap(1, 1)		# Thanks to NS4Y
@@ -151,10 +156,10 @@ class FrequencyDisplay(wx.lib.stattext.GenStaticText):
     self.SetLabel('%s Hz' % txt)
   def GetIndex(self, event):		# Determine which digit is being changed
     mouse_x, mouse_y = event.GetPosition()
-    width, height = self.GetClientSizeTuple()
+    width, height = self.GetClientSize().Get()
     text = self.GetLabel()
     tw, th = self.GetTextExtent(text)
-    edge = (width - tw) / 2
+    edge = (width - tw) // 2
     digit = self.GetTextExtent('0')[0]
     blank = self.GetTextExtent(' ')[0]
     if mouse_x < edge - digit:
@@ -169,7 +174,7 @@ class FrequencyDisplay(wx.lib.stattext.GenStaticText):
       x -= digit * 3 + blank
     if x < 0:
       return None
-    return x / digit + shift * 3	# index of digit being changed
+    return x // digit + shift * 3	# index of digit being changed
   def OnLeftDown(self, event):		# Click on a digit changes the frequency
     if self.repeat_time:
       self.timer.Stop()
@@ -178,8 +183,8 @@ class FrequencyDisplay(wx.lib.stattext.GenStaticText):
     if index is not None:
       self.index = index
       mouse_x, mouse_y = event.GetPosition()
-      width, height = self.GetClientSizeTuple()
-      if mouse_y < height / 2:
+      width, height = self.GetClientSize().Get()
+      if mouse_y < height // 2:
         self.increase = True
       else:
         self.increase = False
@@ -239,10 +244,18 @@ class SliderBoxH:
       w2, h2 = self.text_ctrl.GetSize()
       self.text_ctrl.SetSizeHints(w2, -1, w2)
       self.slider = wx.Slider(parent, -1, init, themin, themax)
-    else:
+    else:	# Thanks to Stephen Hurd
       self.text_ctrl = wx.StaticText(parent, -1, t, pos=pos)
       w2, h2 = self.text_ctrl.GetSize()
-      self.slider = wx.Slider(parent, -1, init, themin, themax, (pos[0] + w2, pos[1]), (width - w2, h2))
+      self.slider = wx.Slider(parent, -1, init, themin, themax)
+      w3, h3 = self.slider.GetSize()
+      p2 = pos[1]
+      if h3 > h2:
+        p2 -= (h3 - h2) / 2
+      else:
+        p2 += (h2 - h3) / 2
+      self.slider.SetSize((width - w2, h3))
+      self.slider.SetPosition((pos[0] + w2, p2))
     self.slider.Bind(wx.EVT_SCROLL, self.OnScroll)
     self.text_ctrl.SetForegroundColour(parent.GetForegroundColour())
     self.OnScroll()
@@ -346,7 +359,7 @@ class QuiskText1(wx.lib.stattext.GenStaticText):
     self.SetFont(font)
   def OnPaint(self, event):
     dc = wx.PaintDC(self)
-    width, height = self.GetClientSize()
+    width, height = self.GetClientSize().Get()
     if not width or not height:
       return
     dc.SetPen(self.pen)
@@ -371,7 +384,7 @@ class QuiskText1(wx.lib.stattext.GenStaticText):
     if style & wx.ALIGN_RIGHT:
       x = width - w - 4
     elif style & wx.ALIGN_CENTER:
-      x = (width - w - 1)/2
+      x = (width - w - 1)//2
     else:
       x = 3
     dc.DrawText(label, x, y)
@@ -396,6 +409,12 @@ class QuiskText(wx.BoxSizer):
 
 class QuiskButtons:
   """Base class for special buttons."""
+  def __init__(self):
+    self.up_brush = wx.Brush(conf.color_btn)
+    r, g, b = self.up_brush.GetColour().Get(False)
+    r, g, b = min(255,r+32), min(255,g+32), min(255,b+32)
+    self.down_brush = wx.Brush(wx.Colour(r, g, b))
+    self.color_disable = conf.color_disable
   def InitButtons(self, text, text_color=None):
     if text_color:
       self.text_color = text_color
@@ -405,6 +424,7 @@ class QuiskButtons:
     self.SetBackgroundColour(conf.color_btn)
     self.SetUseFocusIndicator(False)
     self.decoration = None
+    self.char_shortcut = ''
     self.SetFont(button_font)
     if text:
       w, h = self.GetTextExtent(text)
@@ -416,21 +436,42 @@ class QuiskButtons:
     h += button_bezel * 2
     self.SetSizeHints(w, h, 999, h, 1, 1)
   def DrawLabel(self, dc, width, height, dx=0, dy=0):	# Override to change Disable text color
+      if self.up:	# Clear the background here
+        dc.SetBrush(self.up_brush)
+      else:
+        dc.SetBrush(self.down_brush)
+      dc.SetPen(wx.TRANSPARENT_PEN)
+      bw = self.bezelWidth
+      dc.DrawRectangle(bw, bw, width - bw * 2, height - bw * 2)
       dc.SetFont(self.GetFont())
       label = self.GetLabel()
       tw, th = dc.GetTextExtent(label)
       self.label_width = tw
       dx = dy = self.labelDelta
-      slabel = re.split('('+unichr(0x25CF)+')', label)	# unicode symbol for record: a filled dot
+      slabel = re.split('('+u'\u25CF'+')', label)	# unicode symbol for record: a filled dot
       for part in slabel:		# This code makes the symbol red.  Thanks to Christof, DJ4CM.
         if self.IsEnabled():
-          if part == unichr(0x25CF):
+          if part == u'\u25CF':
             dc.SetTextForeground('red')
           else:
             dc.SetTextForeground(self.text_color)
         else:
-          dc.SetTextForeground(conf.color_disable)
-        dc.DrawText(part, (width-tw)//2+dx, (height-th)//2+dy)
+          dc.SetTextForeground(self.color_disable)
+        if self.char_shortcut:
+          scut = part.split(self.char_shortcut, 1)
+          if len(scut) == 2:	# The shortcut character is present in the string
+            dc.DrawText(scut[0], (width-tw)//2+dx, (height-th)//2+dy)
+            dx += dc.GetTextExtent(scut[0])[0]
+            dc.SetFont(button_uline_font)
+            dc.DrawText(self.char_shortcut, (width-tw)//2+dx, (height-th)//2+dy)
+            dx += dc.GetTextExtent(self.char_shortcut)[0]
+            dc.SetFont(self.GetFont())
+            dc.DrawText(scut[1], (width-tw)//2+dx, (height-th)//2+dy)
+            dx += dc.GetTextExtent(scut[1])[0]
+          else:
+            dc.DrawText(part, (width-tw)//2+dx, (height-th)//2+dy)
+        else:
+          dc.DrawText(part, (width-tw)//2+dx, (height-th)//2+dy)
         dx += dc.GetTextExtent(part)[0]
       if self.decoration and conf.decorate_buttons:
         wd, ht = dc.GetTextExtent(self.decoration)
@@ -450,6 +491,13 @@ class QuiskButtons:
       dc.DrawText(uch, width - wd, (height - ht) // 2)
     else:
       dc.DrawText(uch, width - wd * 15 // 10, (height - ht) // 2)
+  def SetColorGray(self):
+    self.SetBackgroundColour(wx.Colour(220, 220, 220))	# This sets the bezel colors
+    self.SetBezelWidth(2)
+    self.text_color = 'black'
+    self.color_disable = 'white'
+    self.up_brush = wx.Brush(wx.Colour(220, 220, 220))
+    self.down_brush = wx.Brush(wx.Colour(240, 240, 240))
 
 class QuiskBitmapButton(wx.lib.buttons.GenBitmapButton):
   def __init__(self, parent, command, bitmap, use_right=False):
@@ -482,6 +530,7 @@ class QuiskBitmapButton(wx.lib.buttons.GenBitmapButton):
 class QuiskPushbutton(QuiskButtons, wx.lib.buttons.GenButton):
   """A plain push button widget."""
   def __init__(self, parent, command, text, use_right=False, text_color=None, style=0):
+    QuiskButtons.__init__(self)
     wx.lib.buttons.GenButton.__init__(self, parent, -1, text, style=style)
     self.command = command
     self.Bind(wx.EVT_BUTTON, self.OnButton)
@@ -504,6 +553,7 @@ class QuiskPushbutton(QuiskButtons, wx.lib.buttons.GenButton):
 class QuiskRepeatbutton(QuiskButtons, wx.lib.buttons.GenButton):
   """A push button that repeats when held down."""
   def __init__(self, parent, command, text, up_command=None, use_right=False):
+    QuiskButtons.__init__(self)
     wx.lib.buttons.GenButton.__init__(self, parent, -1, text)
     self.command = command
     self.up_command = up_command
@@ -558,15 +608,16 @@ class QuiskCheckbutton(QuiskButtons, wx.lib.buttons.GenToggleButton):
   """A button that pops up and down, and changes color with each push."""
   # Check button; get the checked state with self.GetValue()
   def __init__(self, parent, command, text, color=None, use_right=False):
+    QuiskButtons.__init__(self)
     wx.lib.buttons.GenToggleButton.__init__(self, parent, -1, text)
     self.InitButtons(text)
     self.Bind(wx.EVT_BUTTON, self.OnButton)
     self.button_down = 0		# used for radio buttons
     self.command = command
     if color is None:
-      self.color = conf.color_check_btn
+      self.down_brush = wx.Brush(conf.color_check_btn)
     else:
-      self.color = color
+      self.down_brush = wx.Brush(color)
     self.direction = 1
     if use_right:
       self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
@@ -574,19 +625,11 @@ class QuiskCheckbutton(QuiskButtons, wx.lib.buttons.GenToggleButton):
   def SetValue(self, value, do_cmd=False):
     wx.lib.buttons.GenToggleButton.SetValue(self, value)
     self.button_down = value
-    if value:
-      self.SetBackgroundColour(self.color)
-    else:
-      self.SetBackgroundColour(conf.color_btn)
     if do_cmd and self.command:
       event = wx.PyEvent()
       event.SetEventObject(self)
       self.command(event)
   def OnButton(self, event):
-    if self.GetValue():
-      self.SetBackgroundColour(self.color)
-    else:
-      self.SetBackgroundColour(conf.color_btn)
     if self.command:
       self.command(event)
   def OnRightDown(self, event):
@@ -595,6 +638,8 @@ class QuiskCheckbutton(QuiskButtons, wx.lib.buttons.GenToggleButton):
   def OnRightUp(self, event):
     self.OnLeftUp(event)
     self.direction = 1
+  def Shortcut(self, event):
+    self.SetValue(not self.GetValue(), True)
 
 class QuiskBitField(wx.Window):
   """A control used to set/unset bits."""
@@ -691,7 +736,7 @@ class QFilterButtonWindow(wx.Frame):
     except ValueError:
       index = 0
       self.wrap.button.slider_value = self.valuelist[0]
-    self.slider = wx.Slider(self, -1, index, 0, 100, (0, 0), (w/2, height), wx.SL_VERTICAL|wx.SL_INVERSE)
+    self.slider = wx.Slider(self, -1, index, 0, 100, (0, 0), (w//2, height), wx.SL_VERTICAL|wx.SL_INVERSE)
     self.slider.Bind(wx.EVT_SCROLL, self.OnSlider)
     self.SetTitle("%d" % self.valuelist[index])
     self.Show()
@@ -727,7 +772,7 @@ class QSliderButtonWindow(wx.Frame):
     self.Bind(wx.EVT_CLOSE, self.OnClose)
     self.slider = wx.Slider(self, -1, value,
              self.button.slider_min, self.button.slider_max,
-             (0, 0), (w/2, height), wx.SL_VERTICAL|wx.SL_INVERSE)
+             (0, 0), (w//2, height), wx.SL_VERTICAL|wx.SL_INVERSE)
     self.slider.Bind(wx.EVT_SCROLL, self.OnSlider)
     if self.button.display:
       value = float(value) / self.button.slider_max
@@ -752,7 +797,7 @@ class QDualSliderButtonWindow(wx.Frame):	# Thanks to Steve, KF7O
     x, y = button.GetPosition().Get()
     x, y = button.GetParent().ClientToScreen(wx.Point(x, y))
     w, h = button.GetSize()
-    w = w * 12 / 10
+    w = w * 12 // 10
     height = h * 10
     size = (w, height)
     if sys.platform == 'win32':
@@ -768,12 +813,12 @@ class QDualSliderButtonWindow(wx.Frame):	# Thanks to Steve, KF7O
     hbox = wx.BoxSizer(wx.HORIZONTAL)
     self.lslider = wx.Slider(panel, -1, self.button.lslider_value,
              self.button.slider_min, self.button.slider_max,
-             (0, 0), (w/2, height), wx.SL_VERTICAL|wx.SL_INVERSE)
+             (0, 0), (w//2, height), wx.SL_VERTICAL|wx.SL_INVERSE)
     self.lslider.Bind(wx.EVT_SCROLL, self.OnSlider)
     hbox.Add(self.lslider, flag=wx.LEFT)
     self.rslider = wx.Slider(panel, -1, self.button.rslider_value,
              self.button.slider_min, self.button.slider_max,
-             (0, 0), (w/2, height), wx.SL_VERTICAL|wx.SL_INVERSE)
+             (0, 0), (w//2, height), wx.SL_VERTICAL|wx.SL_INVERSE)
     self.rslider.Bind(wx.EVT_SCROLL, self.OnSlider)
     hbox.Add(self.rslider, flag=wx.RIGHT)
     panel.SetSizer(hbox)
@@ -857,6 +902,10 @@ class WrapSlider(WrapControl):
     if self.adjust:
       self.adjust.Destroy()
       self.adjust = None
+  def DeleteSliderWindow(self):
+    if self.adjust:
+      self.adjust.Destroy()
+      self.adjust = None
   def OnPopButton(self, event):
     if self.adjust:
       self.adjust.Destroy()
@@ -1002,8 +1051,13 @@ class QuiskCycleCheckbutton(QuiskCheckbutton):
       if self.command:
         self.command(event)
   def DrawLabel(self, dc, width, height, dx=0, dy=0):
-    QuiskButtons.DrawLabel(self, dc, width, height, dx, dy)
+    QuiskCheckbutton.DrawLabel(self, dc, width, height, dx, dy)
     self.DrawGlyphCycle(dc, width, height)
+  def Shortcut(self, event):
+    index = self.index + 1
+    if index >= len(self.labels):
+      index = 0
+    self.SetIndex(index, True)
 
 class RadioButtonGroup:
   """This class encapsulates a group of radio buttons.  This class is not a button!
@@ -1012,23 +1066,31 @@ class RadioButtonGroup:
   of labels can be a list/tuple, and the corresponding button will
   be a cycle button.
   """
-  def __init__(self, parent, command, labels, default):
+  def __init__(self, parent, command, labels, default, shortcuts=()):
     self.command = command
     self.buttons = []
     self.button = None
+    self.shortcuts = list(shortcuts[:])
+    self.last_shortcut = 0
+    i = 0
     for text in labels:
-      if type(text) in (ListType, TupleType):
+      if isinstance(text, (list, tuple)):
         b = QuiskCycleCheckbutton(parent, self.OnButton, text, is_radio=True)
+        if shortcuts:
+          b.char_shortcut = shortcuts[i]
         for t in text:
           if t == default and self.button is None:
             b.SetLabel(t)
             self.button = b
       else:
         b = QuiskCheckbutton(parent, self.OnButton, text)
+        if shortcuts:
+          b.char_shortcut = shortcuts[i]
         if text == default and self.button is None:
           b.SetValue(True)
           self.button = b
       self.buttons.append(b)
+      i += 1
   def ReplaceButton(self, index, button):	# introduce a specialized button
     b = self.buttons[index]
     b.Destroy()
@@ -1082,7 +1144,29 @@ class RadioButtonGroup:
   def GetSelectedButton(self):		# return the selected button
     return self.button
   def GetIndex(self):	# Careful.  Some buttons are complex.
+    if not self.button:
+      return None
     return self.buttons.index(self.button)
+  def Shortcut(self, event):
+    # Multiple buttons can have the same shortcut, so move to the next one.
+    index = self.last_shortcut + 1
+    length = len(self.shortcuts)
+    if index >= length:
+      index = 0
+    for i in range(length):
+      shortcut = self.shortcuts[index]
+      if shortcut and wx.GetKeyState(ord(shortcut)):
+        break
+      index += 1
+      if index >= length:
+        index = 0
+    else:
+      return
+    self.last_shortcut = index
+    button = self.buttons[index]
+    event = wx.PyEvent()
+    event.SetEventObject(button)
+    button.OnButton(event)
 
 class _PopWindow(wx.PopupWindow):
   def __init__(self, parent, command, labels, default):
@@ -1110,7 +1194,7 @@ class RadioBtnPopup:
     self.button_data = {}
     labels = []
     for item in in_labels:
-      if type(item) in (ListType, TupleType):
+      if isinstance(item, (list, tuple)):
         labels.append(item[0])
         self.button_data[item[0]] = [_bitmap_cyclepop, 0, len(item)]	# bitmap, index, max_index
       else:
@@ -1119,7 +1203,7 @@ class RadioBtnPopup:
     self.RbDialog.Hide()
     self.pop_control = wx.BoxSizer(wx.HORIZONTAL)
     self.first_button = QuiskPushbutton(parent, self.OnFirstButton, labels[0], text_color=conf.color_popchoice)
-    self.first_button.decoration = unichr(0x21D2)
+    self.first_button.decoration = u'\u21D2'
     self.second_button = QuiskBitmapButton(parent, self.OnSecondButton, _bitmap_menupop, use_right=True)
     self.pop_control.Add(self.first_button, 1, flag=wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL)
     self.pop_control.Add(self.second_button, 0, flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL|wx.LEFT, border=2)
@@ -1254,8 +1338,8 @@ class FreqSetter(wx.TextCtrl):
     w, h = b.GetSize().Get()
     self.end_pos = (x + w, y + h)
     b.Bind(wx.EVT_SPIN, self.OnSpin)	# The spin button frequencies are in kHz
-    b.SetMin(fmin / 1000)
-    b.SetMax(fmax / 1000)
+    b.SetMin(fmin // 1000)
+    b.SetMax(fmax // 1000)
     self.SetValue(freq)
   def OnText(self, event):
     self.SetBackgroundColour('pink')
@@ -1282,7 +1366,7 @@ class FreqSetter(wx.TextCtrl):
       freq = self.fmin
     elif freq > self.fmax:
       freq = self.fmax
-    self.butn.SetValue(freq / 1000)
+    self.butn.SetValue(freq // 1000)
     txt = FreqFormatter(freq)
     wx.TextCtrl.SetValue(self, txt)
     self.SetBackgroundColour(conf.color_entry)
